@@ -27,6 +27,7 @@
 #endif
 
 #include <string.h>
+#include <girepository.h>
 
 #ifdef OS_OSX
 #include <Carbon/Carbon.h>
@@ -42,7 +43,7 @@
 
 /**
  * SECTION:peas-gtk-plugin-manager
- * @short_description: Management GGtk for plugins.
+ * @short_description: Management GUI for plugins.
  *
  * The #PeasGtkPluginManager is a widget that can be used to manage plugins,
  * i.e. load or unload them, and see some pieces of information.
@@ -71,10 +72,11 @@ struct _PeasGtkPluginManagerPrivate {
 /* Properties */
 enum {
   PROP_0,
-  PROP_ENGINE
+  PROP_ENGINE,
+  PROP_VIEW
 };
 
-G_DEFINE_TYPE (PeasGtkPluginManager, peas_gtk_plugin_manager, GTK_TYPE_VBOX);
+G_DEFINE_TYPE (PeasGtkPluginManager, peas_gtk_plugin_manager, GTK_TYPE_BOX);
 
 static gboolean
 plugin_is_configurable (PeasGtkPluginManager *pm,
@@ -221,7 +223,7 @@ show_configure_cb (GtkWidget            *widget,
   exten = peas_engine_create_extension (pm->priv->engine, info, PEAS_GTK_TYPE_CONFIGURABLE, NULL);
   g_return_if_fail (PEAS_IS_EXTENSION (exten));
 
-  peas_extension_call (exten, "create_configure_widget", &conf_widget);
+  conf_widget = peas_gtk_configurable_create_configure_widget (PEAS_GTK_CONFIGURABLE (exten));
   g_object_unref (exten);
 
   g_return_if_fail (GTK_IS_WIDGET (conf_widget));
@@ -231,7 +233,7 @@ show_configure_cb (GtkWidget            *widget,
 
   conf_dlg = gtk_dialog_new_with_buttons (peas_plugin_info_get_name (info),
                                           toplevel,
-                                          GTK_DIALOG_NO_SEPARATOR,
+                                          0,
                                           GTK_STOCK_CLOSE,
                                           GTK_RESPONSE_CLOSE,
                                           NULL);
@@ -241,13 +243,14 @@ show_configure_cb (GtkWidget            *widget,
 
   if (peas_plugin_info_get_help_uri (info) != NULL)
     {
-      GtkWidget *buttonbox;
+      GtkWidget *hbuttonbox;
       GtkWidget *help_button;
 
-      buttonbox = gtk_dialog_get_action_area (GTK_DIALOG (conf_dlg));
+      hbuttonbox = gtk_dialog_get_action_area (GTK_DIALOG (conf_dlg));
       help_button = gtk_button_new_from_stock (GTK_STOCK_HELP);
-      gtk_container_add (GTK_CONTAINER (buttonbox), help_button);
-      gtk_button_box_set_child_secondary (GTK_BUTTON_BOX (buttonbox), help_button, TRUE);
+      gtk_container_add (GTK_CONTAINER (hbuttonbox), help_button);
+      gtk_button_box_set_child_secondary (GTK_BUTTON_BOX (hbuttonbox),
+                                          help_button, TRUE);
 
       g_signal_connect (help_button,
                         "clicked",
@@ -293,21 +296,7 @@ plugin_loaded_toggled_cb (PeasEngine           *engine,
 }
 
 static void
-selection_changed_cb (GtkTreeSelection    *selection,
-                      PeasGtkPluginManager *pm)
-{
-  PeasGtkPluginManagerView *view;
-  PeasPluginInfo *info;
-
-  view = PEAS_GTK_PLUGIN_MANAGER_VIEW (pm->priv->view);
-  info = peas_gtk_plugin_manager_view_get_selected_plugin (view);
-
-  update_button_sensitivity (pm, info);
-}
-
-static void
-cursor_changed_cb (GtkTreeView         *tree_view,
-                   PeasGtkPluginManager *pm)
+selection_changed_cb (PeasGtkPluginManager *pm)
 {
   PeasGtkPluginManagerView *view;
   PeasPluginInfo *info;
@@ -325,24 +314,18 @@ populate_popup_cb (PeasGtkPluginManagerView *view,
 {
   PeasPluginInfo *info;
   GtkWidget *item;
-  GtkWidget *image;
 
   info = peas_gtk_plugin_manager_view_get_selected_plugin (view);
 
   if (info == NULL)
     return;
 
-  item = gtk_image_menu_item_new_with_mnemonic (_("C_onfigure"));
-  image = gtk_image_new_from_stock (GTK_STOCK_PREFERENCES,
-                                    GTK_ICON_SIZE_MENU);
-  gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (item), image);
+  item = gtk_image_menu_item_new_from_stock (GTK_STOCK_PREFERENCES, NULL);
   g_signal_connect (item, "activate", G_CALLBACK (show_configure_cb), pm);
   gtk_widget_set_sensitive (item, plugin_is_configurable (pm, info));
   gtk_menu_shell_prepend (GTK_MENU_SHELL (menu), item);
 
-  item = gtk_image_menu_item_new_with_mnemonic (_("_About"));
-  image = gtk_image_new_from_stock (GTK_STOCK_ABOUT, GTK_ICON_SIZE_MENU);
-  gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (item), image);
+  item = gtk_image_menu_item_new_from_stock (GTK_STOCK_ABOUT, NULL);
   g_signal_connect (item, "activate", G_CALLBACK (show_about_cb), pm);
   gtk_menu_shell_prepend (GTK_MENU_SHELL (menu), item);
 }
@@ -350,33 +333,33 @@ populate_popup_cb (PeasGtkPluginManagerView *view,
 static void
 peas_gtk_plugin_manager_init (PeasGtkPluginManager *pm)
 {
-  GtkWidget *label;
   GtkWidget *hbuttonbox;
 
   pm->priv = G_TYPE_INSTANCE_GET_PRIVATE (pm,
                                           PEAS_GTK_TYPE_PLUGIN_MANAGER,
                                           PeasGtkPluginManagerPrivate);
 
+  /* If we are using a PeasGtkPluginManager, we know for sure we will be using
+     libpeas-gtk, so let's load the typelib for it here. */
+  g_irepository_require (g_irepository_get_default (),
+                         "PeasGtk", "1.0", 0, NULL);
+
   gtk_box_set_spacing (GTK_BOX (pm), 6);
+  gtk_orientable_set_orientation (GTK_ORIENTABLE (pm),
+                                  GTK_ORIENTATION_VERTICAL);
 
   gtk_widget_push_composite_child ();
 
-  label = gtk_label_new_with_mnemonic (_("_Plugins:"));
-  gtk_label_set_justify (GTK_LABEL (label), GTK_JUSTIFY_LEFT);
-  gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
-  gtk_box_pack_start (GTK_BOX (pm), label, FALSE, TRUE, 0);
-
   pm->priv->sw = gtk_scrolled_window_new (NULL, NULL);
   gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (pm->priv->sw),
                                   GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
   gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (pm->priv->sw),
                                        GTK_SHADOW_IN);
-  gtk_label_set_mnemonic_widget (GTK_LABEL (label), pm->priv->sw);
   gtk_box_pack_start (GTK_BOX (pm), pm->priv->sw, TRUE, TRUE, 0);
 
-  hbuttonbox = gtk_hbutton_box_new ();
-  gtk_button_box_set_layout (GTK_BUTTON_BOX (hbuttonbox), GTK_BUTTONBOX_END);
+  hbuttonbox = gtk_button_box_new (GTK_ORIENTATION_HORIZONTAL);
   gtk_box_set_spacing (GTK_BOX (hbuttonbox), 6);
+  gtk_button_box_set_layout (GTK_BUTTON_BOX (hbuttonbox), GTK_BUTTONBOX_END);
   gtk_box_pack_start (GTK_BOX (pm), hbuttonbox, FALSE, FALSE, 0);
 
   pm->priv->about_button = gtk_button_new_from_stock (GTK_STOCK_ABOUT);
@@ -412,7 +395,9 @@ peas_gtk_plugin_manager_set_property (GObject      *object,
     {
     case PROP_ENGINE:
       pm->priv->engine = g_value_get_object (value);
-      g_object_ref (pm->priv->engine);
+      break;
+    case PROP_VIEW:
+      pm->priv->view = g_value_get_object (value);
       break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -433,6 +418,9 @@ peas_gtk_plugin_manager_get_property (GObject    *object,
     case PROP_ENGINE:
       g_value_set_object (value, pm->priv->engine);
       break;
+    case PROP_VIEW:
+      g_value_set_object (value, peas_gtk_plugin_manager_get_view (pm));
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -445,15 +433,59 @@ peas_gtk_plugin_manager_constructed (GObject *object)
   PeasGtkPluginManager *pm = PEAS_GTK_PLUGIN_MANAGER (object);
   GtkTreeSelection *selection;
 
+  if (pm->priv->engine == NULL)
+    pm->priv->engine = peas_engine_get_default ();
+
+  g_object_ref (pm->priv->engine);
+
   /* When we create the manager, we always rescan the plugins directory
-     Must come after the view has connected to notify::plugin-list */
+   * Must come after the view has connected to notify::plugin-list
+   */
   peas_engine_rescan_plugins (pm->priv->engine);
 
-  pm->priv->view = peas_gtk_plugin_manager_view_new (pm->priv->engine);
+  /* For the view to behave as expected, we must ensure it uses the same
+   * engine as the manager itself.
+   */
+  if (pm->priv->view != NULL)
+    {
+      PeasEngine *engine;
+
+      g_object_get (pm->priv->view,
+                    "engine", &engine,
+                    NULL);
+
+      g_warn_if_fail (engine == pm->priv->engine);
+
+      if (engine != pm->priv->engine)
+        {
+          g_object_unref (pm->priv->view);
+          pm->priv->view = NULL;
+        }
+
+      g_object_unref (engine);
+    }
+
+  if (pm->priv->view == NULL)
+    pm->priv->view = peas_gtk_plugin_manager_view_new (pm->priv->engine);
+
+  gtk_widget_push_composite_child ();
   gtk_container_add (GTK_CONTAINER (pm->priv->sw), pm->priv->view);
+  gtk_widget_pop_composite_child ();
 
   selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (pm->priv->view));
 
+  g_signal_connect_swapped (selection,
+                            "changed",
+                            G_CALLBACK (selection_changed_cb),
+                            pm);
+  g_signal_connect_swapped (pm->priv->view,
+                            "cursor-changed",
+                            G_CALLBACK (selection_changed_cb),
+                            pm);
+  g_signal_connect (pm->priv->view,
+                    "populate-popup",
+                    G_CALLBACK (populate_popup_cb),
+                    pm);
   g_signal_connect_after (pm->priv->engine,
                           "load-plugin",
                           G_CALLBACK (plugin_loaded_toggled_cb),
@@ -462,21 +494,9 @@ peas_gtk_plugin_manager_constructed (GObject *object)
                           "unload-plugin",
                           G_CALLBACK (plugin_loaded_toggled_cb),
                           pm);
-  g_signal_connect (selection,
-                    "changed",
-                    G_CALLBACK (selection_changed_cb),
-                    pm);
-  g_signal_connect (pm->priv->view,
-                    "cursor-changed",
-                    G_CALLBACK (cursor_changed_cb),
-                    pm);
-  g_signal_connect (pm->priv->view,
-                    "populate-popup",
-                    G_CALLBACK (populate_popup_cb),
-                    pm);
 
   /* Update the button sensitivity */
-  selection_changed_cb (selection, pm);
+  selection_changed_cb (pm);
 
   if (G_OBJECT_CLASS (peas_gtk_plugin_manager_parent_class)->constructed != NULL)
     G_OBJECT_CLASS (peas_gtk_plugin_manager_parent_class)->constructed (object);
@@ -511,7 +531,7 @@ peas_gtk_plugin_manager_class_init (PeasGtkPluginManagerClass *klass)
   object_class->dispose = peas_gtk_plugin_manager_dispose;
 
   /**
-   * PeasGtkPLuginManager:engine:
+   * PeasGtkPluginManager:engine:
    *
    * The #PeasEngine this manager is attached to.
    */
@@ -525,21 +545,38 @@ peas_gtk_plugin_manager_class_init (PeasGtkPluginManagerClass *klass)
                                                         G_PARAM_CONSTRUCT_ONLY |
                                                         G_PARAM_STATIC_STRINGS));
 
+  /**
+   * PeasGtkPluginManager:view:
+   *
+   * The #PeasGtkPluginManagerView shown in the #PeasGtkPluginManager.
+   */
+  g_object_class_install_property (object_class,
+                                   PROP_VIEW,
+                                   g_param_spec_object ("view",
+                                                        "view",
+                                                        "The view shown in the manager",
+                                                        PEAS_GTK_TYPE_PLUGIN_MANAGER_VIEW,
+                                                        G_PARAM_READWRITE |
+                                                        G_PARAM_CONSTRUCT_ONLY |
+                                                        G_PARAM_STATIC_STRINGS));
+
   g_type_class_add_private (object_class, sizeof (PeasGtkPluginManagerPrivate));
 }
 
 /**
  * peas_gtk_plugin_manager_new:
- * @engine: A #PeasEngine.
+ * @engine: (allow-none): A #PeasEngine, or %NULL.
  *
  * Creates a new plugin manager for the given #PeasEngine.
  *
+ * If @engine is %NULL, then the default engine will be used.
+ *
  * Returns: the new #PeasGtkPluginManager.
  */
 GtkWidget *
 peas_gtk_plugin_manager_new (PeasEngine *engine)
 {
-  g_return_val_if_fail (PEAS_IS_ENGINE (engine), NULL);
+  g_return_val_if_fail (engine == NULL || PEAS_IS_ENGINE (engine), NULL);
 
   return GTK_WIDGET (g_object_new (PEAS_GTK_TYPE_PLUGIN_MANAGER,
                                    "engine", engine,
@@ -548,11 +585,11 @@ peas_gtk_plugin_manager_new (PeasEngine *engine)
 
 /**
  * peas_gtk_plugin_manager_get_view:
- * @pm: A @PeasGtkPluginManager.
+ * @pm: A #PeasGtkPluginManager.
  *
  * Returns the #PeasGtkPluginManagerView of @pm.
  *
- * Returns: the view of @pm.
+ * Returns: (transfer none): the #GtkTreeView of @pm.
  */
 GtkWidget *
 peas_gtk_plugin_manager_get_view (PeasGtkPluginManager *pm)
