@@ -74,7 +74,9 @@
  *
  *   set = peas_extension_set_new (engine, PEAS_TYPE_ACTIVATABLE,
  *                                 "object", window, NULL);
- *   peas_extension_set_call (set, "activate");
+ *   peas_extension_set_foreach (set,
+ *                               (PeasExtensionSetForeachFunc) on_extension_added,
+ *                               NULL);
  *   g_signal_connect (set, "extension-added",
  *                     G_CALLBACK (on_extension_added), NULL);
  *   g_signal_connect (set, "extension-removed",
@@ -468,12 +470,14 @@ peas_extension_set_get_extension (PeasExtensionSet *set,
  * peas_extension_set_call: (skip)
  * @set: A #PeasExtensionSet.
  * @method_name: the name of the method that should be called.
- * @Varargs: arguments for the method.
+ * @...: arguments for the method.
  *
  * Call a method on all the #PeasExtension instances contained in @set.
  *
  * See peas_extension_call() for more information.
  *
+ * Deprecated: 1.2. Use peas_extension_set_foreach() instead.
+ *
  * Return value: %TRUE on successful call.
  */
 gboolean
@@ -504,6 +508,8 @@ peas_extension_set_call (PeasExtensionSet *set,
  *
  * See peas_extension_call_valist() for more information.
  *
+ * Deprecated: 1.2. Use peas_extension_set_foreach() instead.
+ *
  * Return value: %TRUE on successful call.
  */
 gboolean
@@ -547,6 +553,8 @@ peas_extension_set_call_valist (PeasExtensionSet *set,
  *
  * Return value: %TRUE on successful call.
  *
+ * Deprecated: 1.2. Use peas_extension_set_foreach() instead.
+ *
  * Rename to: peas_extension_set_call
  */
 gboolean
@@ -563,6 +571,34 @@ peas_extension_set_callv (PeasExtensionSet *set,
   return klass->call (set, method_name, args);
 }
 
+/**
+ * peas_extension_set_foreach:
+ * @set: A #PeasExtensionSet.
+ * @func: (scope call): A function call for each extension.
+ * @data: Optional data to be passed to the function or %NULL.
+ *
+ * Calls @func for each #PeasExtension.
+ *
+ * Since: 1.2
+ */
+void
+peas_extension_set_foreach (PeasExtensionSet            *set,
+                            PeasExtensionSetForeachFunc  func,
+                            gpointer                     data)
+{
+  GList *l;
+
+  g_return_if_fail (PEAS_IS_EXTENSION_SET (set));
+  g_return_if_fail (func != NULL);
+
+  for (l = set->priv->extensions; l; l = l->next)
+    {
+      ExtensionItem *item = (ExtensionItem *) l->data;
+
+      func (set, item->info, item->exten, data);
+    }
+}
+
 /**
  * peas_extension_set_newv:
  * @engine: (allow-none): A #PeasEngine, or %NULL.
@@ -654,7 +690,7 @@ peas_extension_set_new_valist (PeasEngine  *engine,
  * @engine: A #PeasEngine, or %NULL.
  * @exten_type: the extension #GType.
  * @first_property: the name of the first property.
- * @Varargs: the value of the first property, followed optionally by more
+ * @...: the value of the first property, followed optionally by more
  *   name/value pairs, followed by %NULL.
  *
  * Create a new #PeasExtensionSet for the @exten_type extension type.
