@@ -46,12 +46,9 @@ struct _PeasPluginLoader {
 struct _PeasPluginLoaderClass {
   GObjectClass parent;
 
-  void          (*add_module_directory)   (PeasPluginLoader *loader,
-                                           const gchar      *module_dir);
-
-  gboolean      (*load)                   (PeasPluginLoader *loader,
+  gboolean       (*load)                  (PeasPluginLoader *loader,
                                            PeasPluginInfo   *info);
-  void          (*unload)                 (PeasPluginLoader *loader,
+  void           (*unload)                (PeasPluginLoader *loader,
                                            PeasPluginInfo   *info);
   gboolean       (*provides_extension)    (PeasPluginLoader *loader,
                                            PeasPluginInfo   *info,
@@ -62,14 +59,11 @@ struct _PeasPluginLoaderClass {
                                            guint             n_parameters,
                                            GParameter       *parameters);
 
-  void          (*garbage_collect)        (PeasPluginLoader *loader);
+  void           (*garbage_collect)       (PeasPluginLoader *loader);
 };
 
 GType         peas_plugin_loader_get_type             (void)  G_GNUC_CONST;
 
-void          peas_plugin_loader_add_module_directory (PeasPluginLoader *loader,
-                                                       const gchar      *module_dir);
-
 gboolean      peas_plugin_loader_load                 (PeasPluginLoader *loader,
                                                        PeasPluginInfo   *info);
 void          peas_plugin_loader_unload               (PeasPluginLoader *loader,
