@@ -46,6 +46,8 @@ testing_init (void)
                                  BUILDDIR "/tests/libpeas/introspection",
                                  "Introspection", "1.0", 0, &error);
   g_assert_no_error (error);
+
+  initialized = TRUE;
 }
 
 PeasEngine *
@@ -58,10 +60,8 @@ testing_engine_new (void)
   testing_util_push_log_hook ("*Bad plugin file: *invalid.plugin*");
   testing_util_push_log_hook ("*Error loading *invalid.plugin*");
 
-  testing_util_push_log_hook ("*Could not find 'IAge' in *info-missing-iage.plugin*");
   testing_util_push_log_hook ("*Could not find 'Module' in *info-missing-module.plugin*");
   testing_util_push_log_hook ("*Could not find 'Name' in *info-missing-name.plugin*");
-  testing_util_push_log_hook ("*Error loading *info-missing-iage.plugin*");
   testing_util_push_log_hook ("*Error loading *info-missing-module.plugin*");
   testing_util_push_log_hook ("*Error loading *info-missing-name.plugin*");
 
